@@ -1,8 +1,12 @@
+from __future__ import annotations
+
 import sys
 import json
 import logging
 from pathlib import Path
 from typing import Dict, List, Optional, Union, Type, Any
+from typing_extensions import TypedDict, Literal
+
 from pydantic import Field
 
 from .models import BaseModel
@@ -28,19 +32,42 @@ class Request(BaseModel):
     params: Optional[Dict[str, Any]]
 
 
-class Response(BaseModel):
+class SuccessResponse(BaseModel):
     id: int
     jsonrpc: str = '2.0'
     result: Optional[Dict[str, Any]]
 
 
+class ErrorData(TypedDict):
+    code: int
+    message: str
+    data: object
+
+
+class ErrorResponse(BaseModel):
+    id: int
+    error: ErrorData
+    jsonrpc: str = '2.0'
+
+
+Response = Union[SuccessResponse, ErrorResponse]
+
+EngineType = Literal[
+    'prismaFmt',
+    'queryEngine',
+    'libqueryEngine',
+    'migrationEngine',
+    'introspectionEngine',
+]
+
+
 class Manifest(BaseModel):
     """Generator metadata"""
 
     prettyName: str = Field(alias='name')
     defaultOutput: Union[str, Path] = Field(alias='default_output')
     denylist: Optional[List[str]] = None
-    requiresEngines: Optional[List[str]] = Field(
+    requiresEngines: Optional[List[EngineType]] = Field(
         alias='requires_engines', default=None
     )
     requiresGenerators: Optional[List[str]] = Field(
